// Copyright 2004 Nokia Corporation.
//
// THIS SOURCE CODE IS PROVIDED 'AS IS', WITH NO WARRANTIES WHATSOEVER,
// EXPRESS OR IMPLIED, INCLUDING ANY WARRANTY OF MERCHANTABILITY, FITNESS
// FOR ANY PARTICULAR PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE
// OR TRADE PRACTICE, RELATING TO THE SOURCE CODE OR ANY WARRANTY OTHERWISE
// ARISING OUT OF ANY PROPOSAL, SPECIFICATION, OR SAMPLE AND WITH NO
// OBLIGATION OF NOKIA TO PROVIDE THE LICENSEE WITH ANY MAINTENANCE OR
// SUPPORT. FURTHERMORE, NOKIA MAKES NO WARRANTY THAT EXERCISE OF THE
// RIGHTS GRANTED HEREUNDER DOES NOT INFRINGE OR MAY NOT CAUSE INFRINGEMENT
// OF ANY PATENT OR OTHER INTELLECTUAL PROPERTY RIGHTS OWNED OR CONTROLLED
// BY THIRD PARTIES
//
// Furthermore, information provided in this source code is preliminary,
// and may be changed substantially prior to final release. Nokia Corporation
// retains the right to make changes to this source code at
// any time, without notice. This source code is provided for informational
// purposes only.
//
// Nokia and Nokia Connecting People are registered trademarks of Nokia
// Corporation.
// Java and all Java-based marks are trademarks or registered trademarks of
// Sun Microsystems, Inc.
// Other product and company names mentioned herein may be trademarks or
// trade names of their respective owners.
//
// A non-exclusive, non-transferable, worldwide, limited license is hereby
// granted to the Licensee to download, print, reproduce and modify the
// source code. The licensee has the right to market, sell, distribute and
// make available the source code in original or modified form only when
// incorporated into the programs developed by the Licensee. No other
// license, express or implied, by estoppel or otherwise, to any other
// intellectual property rights is granted herein.
import java.io.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import javax.microedition.lcdui.*;

// This class displays a selected image centered in the screen
class ImageCanvas
        extends Canvas implements CommandListener {

    private final ImageViewerMIDlet midlet;
    private static final int CHUNK_SIZE = 0;
    private Image currentImage = null;
    int imageWidth;
    int imageHeight;
    private Command backCommand = new Command("Back", Command.BACK, 0);
    private Ticker ticker = new Ticker("Image Viewer");
    private int mouseDownX;
    private int mouseDownY;
    private int deltaX;
    private int deltaY;
    private int posX;
    private int posY;

    ImageCanvas(ImageViewerMIDlet midlet) {
        setTitle("Image Viewer");
        this.midlet = midlet;
        this.addCommand(backCommand);
        this.setCommandListener(this);
        this.setTicker(ticker);
    }

    public boolean displayImage(String imgName) {
        try {
            FileConnection fileConn =
                    (FileConnection) Connector.open(imgName, Connector.READ);

            InputStream fis = fileConn.openInputStream();
            int overallSize = (int) fileConn.fileSize();
            
            byte[] imageData = new byte[overallSize];

            int readAmount = fis.read(imageData, 0, overallSize);

            fis.close();
            fileConn.close();
            ticker.setString("Image Viewer:" + imgName);
            currentImage = Image.createImage(imageData, 0, overallSize);
            fis.close();
            fileConn.close();
            imageWidth = currentImage.getWidth();
            imageHeight = currentImage.getHeight();
            repaint();
        } catch (IOException e) {
            midlet.showError(e);
            return false;
        } catch (Exception e) {
            e.printStackTrace();
            midlet.showError(e);
            return false;
        } catch (Error e) {
            e.printStackTrace();
            if (e instanceof OutOfMemoryError) {
                midlet.showError("File is too large to display");
            } else {
                midlet.showError("Failed to display this file. " + e.getMessage());
            }
            return false;
        }
        return true;
    }

    protected void paint(Graphics g) {
        int w = getWidth();
        int h = getHeight();

        // Set background color to black
        g.setColor(0x00000000);
        g.fillRect(0, 0, w, h);

        setImagePlacementPoint();

        System.out.println("Pos X:" + posX + " Y:" + posY);

        if (currentImage != null) {
            g.drawImage(currentImage,
                    posX,
                    posY,
                    Graphics.HCENTER | Graphics.VCENTER);
        } else {
            // If no image is available display a message
            g.setColor(0x00FFFFFF);
            g.drawString("No image",
                    posX,
                    posY,
                    Graphics.HCENTER | Graphics.BASELINE);
        }
    }

    protected void keyReleased(int keyCode) {
        // Exit with any key
        midlet.displayFileBrowser();
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command == backCommand) {
            currentImage = null;
            midlet.displayFileBrowser();
        }
    }

    protected void pointerPressed(int x, int y) {
        mouseDownX = x;
        mouseDownY = y;
    }

    protected void pointerReleased(int x, int y) {
        deltaX = 0;
        deltaY = 0;
        System.out.println("up up up !");
    }

    protected void pointerDragged(int x, int y) {
        deltaX = x - mouseDownX;
        deltaY = y - mouseDownY;
        mouseDownX = x;
        mouseDownY = y;
        repaint();
    }

    void setImagePlacementPoint() {

        // This needs to be taken each time, since the values will be chaged when
        // user tilt the phone to potrait mode to landscape mode.
        int canvasWidth = getWidth();
        int canvasHeight = getHeight();

        if (imageWidth > canvasWidth && deltaX != 0) {
            posX += deltaX;
            if (posX < (canvasWidth - imageWidth / 2)) {
                posX = canvasWidth - imageWidth / 2;
            } else if (posX > (imageWidth / 2)) {
                posX = (imageWidth / 2);
            }
        } else {
            posX = canvasWidth / 2;
        }

        if (imageHeight > canvasHeight && deltaY != 0) {
            posY += deltaY;
            if (posY < (canvasHeight - imageHeight / 2)) {
                posY = canvasHeight - imageHeight / 2;
            } else if (posY > (imageHeight / 2)) {
                posY = (imageHeight / 2);
            }
        } else {
            posY = canvasHeight / 2;
        }
    }
}

